#if ! defined(__LKT_COMMON_H__)
#define __LKT_COMMON_H__

#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>

/* Max value for any buffer, to not squash the stack. */
#define BUFFER_MAX 4096

#ifndef MAX
    #define MAX(a, b) ((a) < (b) ? (b) : (a))
#endif /* MAX */

#ifndef MIN
    #define MIN(a, b) ((a) > (b) ? (b) : (a))
#endif /* MIN */

#define RETURN_IF(cond, msg, ret)       \
    if (cond) {                         \
        LOG_DEBUG("DEBUG", "%s", msg);  \
        return ret;                     \
    }
#define GOTO_IF(cond, msg, label)       \
    if (cond) {                         \
        LOG_DEBUG("DEBUG", "%s", msg);  \
        goto label;                     \
    }

#define GOTO_UNLESS(cond, msg, label)   GOTO_IF(!(cond), msg, label)
#define RETURN_UNLESS(cond, msg, ret)   RETURN_IF(!(cond), msg, ret)
#define NOTHING                         /* Usefull to return nothing in previous macros */

#define STRTOL(ret, str, endptr, err_flag)                  \
{                                                           \
    err_flag = 0;                                           \
    errno    = 0;                                           \
    ret      = str == NULL ? 0 : strtol(str, &(endptr), 0); \
    err_flag = errno != 0 || endptr == str;                 \
}

/* Custom defined assert. */
extern void (*__lkt_assert)(const char *file, int line, const char *func, const char *msg);
#ifdef assert
    #undef assert
#endif
#ifdef NDEBUG
    #define assert(ex)
#else
    #define assert(ex) (void) ((ex) || (__lkt_assert(__FILE__, __LINE__, __func__, #ex), 0))
#endif

/* Custom log functions. */
enum log_level {
    ERROR   = 1,
    WARN    = 2,
    INFO    = 3,
    DEBUG   = 4,
    __LAST_UNUSED_LOG_LEVEL,    /* Usefull to enable all by default */
};
extern enum log_level __log_level;
void __lkt_log(enum log_level, const char *section, const char *func, const char *format, ...);
#define LOG_INFO( section, ...) __lkt_log(INFO,  section, __func__, __VA_ARGS__)
#define LOG_WARN( section, ...) __lkt_log(WARN,  section, __func__, __VA_ARGS__)
#define LOG_ERROR(section, ...) __lkt_log(ERROR, section, __func__, __VA_ARGS__)
#define LOG_DEBUG(section, ...) __lkt_log(DEBUG, section, __func__, __VA_ARGS__)

#define SELF_EXECUTABLE_LINUX       "/proc/self/exe"
#define SELF_EXECUTABLE_FREEBSD     "/proc/curproc/file"
#define SELF_EXECUTABLE_SOLARIS     "/proc/self/path/a.out"

#define LKT_ENV_RESTART             "__LKT_RESTART"
#define LKT_ENV_CURRENT             "__LKT_CURRENT"
#define LKT_ENV_SET_ASSERT          "__LKT_SET_ASSERT"

#define LKT_LINE_MAX                1024
#define LKT_MAX_SQLITE_STATEMENT    1024
#define LKT_PROTECTED_DATABASE      "disk"

#define LKT_DB_ID                   "id"
#define LKT_DB_NAME                 "source_name"
#define LKT_DB_TITLE                "song_title"
#define LKT_DB_CAT                  "category"
#define LKT_DB_TYPE                 "song_type"
#define LKT_DB_AUTHOR               "author_name"
#define LKT_DB_LANG                 "language"
#define LKT_DB_ALL                  "string"

#define LEKTOR_TAG_MAX              256
#define LKT_MESSAGE_ARGS_MAX        32
#define LKT_MESSAGE_MAX             2048
#define LKT_DEFAULT_LIST_SIZE       10

typedef volatile enum {
    MPD_IDLE_NONE               = 0,

    MPD_IDLE_DATABASE           = ( 1 << 1  ),
    MPD_IDLE_UPDATE             = ( 1 << 2  ),
    MPD_IDLE_STORED_PLAYLIST    = ( 1 << 3  ),
    MPD_IDLE_PLAYLIST           = ( 1 << 4  ),
    MPD_IDLE_PLAYER             = ( 1 << 5  ),
    MPD_IDLE_MIXER              = ( 1 << 6  ),
    MPD_IDLE_OUTPUT             = ( 1 << 7  ),
    MPD_IDLE_OPTIONS            = ( 1 << 8  ),
    MPD_IDLE_PARTITION          = ( 1 << 9  ),
    MPD_IDLE_STICKER            = ( 1 << 10 ),
    MPD_IDLE_SUBSCRIPTION       = ( 1 << 11 ),
    MPD_IDLE_MESSAGE            = ( 1 << 12 ),

    MPD_IDLE_ALL =
        MPD_IDLE_DATABASE | MPD_IDLE_UPDATE       | MPD_IDLE_STORED_PLAYLIST  |
        MPD_IDLE_PLAYLIST | MPD_IDLE_PLAYER       | MPD_IDLE_MIXER            |
        MPD_IDLE_OUTPUT   | MPD_IDLE_OPTIONS      | MPD_IDLE_PARTITION        |
        MPD_IDLE_STICKER  | MPD_IDLE_SUBSCRIPTION | MPD_IDLE_MESSAGE,
} mpd_idle_flag;

#define LKT_BACKLOG         32
#define COMMAND_LIST_MAX    64
#define BUFFER_OUT_MAX      16
#define MPD_VERSION         "0.21.16"

#define STR_MATCH(str1, str2)           (! strcasecmp(str1, str2))
#define STR_NMATCH(str1, str2, n)       (! strncasecmp(str1, str2, n))

#define GET_VA_COUNT(...)   PP_NARG_(__VA_ARGS__,PP_RSEQ_N())
#define PP_NARG_(...)       PP_ARG_N(__VA_ARGS__)
#define PP_ARG_N(   _1, _2, _3, _4, _5, _6, _7, _8, _9,_10,_11,_12,_13,_14,     \
                    _15,_16,_17,_18,_19,_20,_21,_22,_23,_24,_25,_26,_27,_28,    \
                    _29,_30,_31,_32,_33,_34,_35,_36,_37,_38,_39,_40,_41,_42,    \
                    _43,_44,_45,_46,_47,_48,_49,_50,_51,_52,_53,_54,_55,_56,    \
                    _57,_58,_59,_60,_61,_62,_63,N,...) N
#define PP_RSEQ_N() 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48,47,46,45,   \
                    44,43,42,41,40,39,38,37,36,35,34,33,32,31,30,29,28,27,26,   \
                    25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,  \
                    4,3,2,1,0

#define not_implemented() __not_implemented(__func__,__FILE__,__LINE__)
extern void __not_implemented(const char *func, char *file, int line);

#define UNUSED(...) __unused((void *) __VA_ARGS__)
void __unused(void *, ...);

long get_mtime(const char *path);

int is_utf8(const char *string);

/* Same as `be_uint64_t` but for Big-endian doubles. The result is a
   correct endian number. Restriction: bytes must be of length 8.
   Use the be_uint64_t defined function. */
double be_double_t(const uint8_t bytes[]);

/* Same as `be_double_t` but for floats. Restriction: bytes must
   be of length 4. The result is a correct endian number Use the
   be_uint32_t defined function. */
float be_float_t(const uint8_t bytes[]);

/* Read `bytes` as the big endian representation of a 32-bit unsigned integer.
   `n` is the number of bytes in `bytes`. The result is a 'correct'-endian.
   Returns 0 if n is 0.
   Restriction: n <= 4 */
uint32_t be_uint32_t(const uint8_t bytes[], size_t n);

/* Same as `be_uint32_t` but for 64-bit unsigned integers. The result is a
   'correct'-endian. Restriction: n <= 8 */
uint64_t be_uint64_t(const uint8_t bytes[], size_t n);

/* Trim the string `str` of the char `c` from the right and the left.
   The string may not be null terminated.
   Returns a pointer to the the trimmed string
   Restrictions: the len of the string must be `len`. */
char *trim(char *str, size_t len, char c);

/* Get a line from stdin safely. */
int get_stdin_line(const char *prompt, char *buf, size_t len);

int read_self_exe(char *path, size_t len);

long long_length(long integer);

/* Safe functions, which preserve the terminal '\0'. */
void *safe_malloc(size_t size);
void *safe_zero_malloc(size_t size);
int safe_snprintf(char *dest, const size_t max_len, const char *format, ...);
char *safe_strncpy(char *dest, const char *src, size_t n);

/* Iterate throught a string, copy each element in the dest string.
   The save ptr must contains a null integer. */
int iter_string(const char *str, const char *sep, char *dest, const size_t dest_len, size_t *save);

#endif /* __LKT_COMMON_H__ */
