#if ! defined(__LKT_CONFIG_H__)
#define __LKT_CONFIG_H__

#include <lektor/common.h>
#include <stddef.h>
#include <sqlite3.h>
#include <stdio.h>

/* Forward definition of the lkt_state structure */
struct lkt_state;

/* The default configuration, stored as a string */
#define section(sct)        "\n[" sct "]\n"
#define value(key, val)     key " = " val "\n"
#define value_opt(key, val) value(key, val)
static const char *const lkt_default_config_file =
#include <lektor/config.def>
    ;
#undef value
#undef value_opt
#undef section

/* It is just an alias to the consecutive use of config_detect_file and
   config_new function, with the return an '&&' between the two returns of the
   two other functions. */
int config_open(volatile sqlite3 *db);

/* Get the path to the config file that may be red, taking into account the
   priority between existing files. The returned path is a path to an existing
   file. If no file is found, returns a non zero value. Returns 1 otherwise. */
int config_detect_file(char *conf, size_t conf_len);

/* Create and read the configuration in the conf file and write it into
   lkt_conf. The type is opaque because it's a C++ class.
   Return 0 if the read operation was a success, a non zero value otherwise. */
int config_new(volatile sqlite3 *db, const char *conf);

/* Prints the default config file. */
void config_default(FILE *output);

/* Get the default user config file, the ~/.config/lektor/lektor.ini file. */
void config_default_file(char *dest, size_t len);

/* Manipulate Environment Variables */
#define env_set(var, val)   setenv(var, val, 1)
#define env_get             getenv

/* Handle everything for a hook */
void config_handle_hook(volatile sqlite3 *db, const char *hook_name);

#endif /* __LKT_CONFIG_H__ */
