#define _POSIX_C_SOURCE 200809L

#include <lektor/common.h>
#include <lektor/reg.h>
#include <lektor/database.h>
#include <lektor/config.h>
#include <lektor/net.h>
#include <dlfcn.h>
#include <stdio.h>
#include <strings.h>
#include <stddef.h>
#include <stdarg.h>
#include <string.h>

struct module_reg *__reg__;

void
reg_export(struct module_reg *reg)
{
    __reg__ = reg;
}

void
reg_global(struct module_reg **reg_ptr)
{
    *reg_ptr = __reg__;
}

void *
__reg_get(struct module_reg *reg, const char *name)
{
    int i;
    for (i = 0; (reg + i)->name && (reg + i)->obj; ++i) {
        if (STR_MATCH((reg + i)->name, name))
            return (reg + i)->obj;
    }
    return NULL;
}

reg_func
__reg_func(struct module_reg *reg, const char *name)
{
    int i;
    for (i = 0; (reg + i)->name && (reg + i)->func; ++i) {
        if (STR_MATCH((reg + i)->name, name))
            return (reg + i)->func;
    }
    return NULL;
}

void *
__from_file(const char *file, void **handle, const char *symbol)
{
    char *error;
    *handle = dlopen(file, RTLD_NOW);

    if (NULL == *handle) {
        LOG_ERROR("REG", "libdl error in dlopen on file '%s': %s",
                  file, dlerror());
        return NULL;
    }

    dlerror();
    void *sym = dlsym(*handle, symbol);

    if ((error = dlerror()) != NULL) {
        LOG_ERROR("REG", "libdl error in dlsym on file '%s': %s\n",
                  file, error);
        return NULL;
    }

    return sym;
}

int
reg_import(const char *mod, struct module_reg **ret, void **handle)
{
    if (strcspn(mod, "/.\'\"\\") != strlen(mod)) {
        LOG_INFO("REG", "Using the dlsym to find '" REG_EXPORT_NAME
                 "' in %s", mod);
        *ret = __from_file(mod, handle, REG_EXPORT_NAME);
    } else {
        LOG_INFO("REG", "Using the reg structure to find %s", mod);
        *ret = __reg_get(__reg__, mod);
        if (!*ret) {
            LOG_INFO("REG", "Not found in reg '" REG_EXPORT_NAME "', using "
                     "the dlsym to find reg '" REG_EXPORT_NAME "' in file %s",
                     mod);
        }
    }
    return *ret != NULL;
}


int
reg_call(struct module_reg *reg, const char *name, int count, ...)
{
    reg_func func = __reg_func(reg, name);
    if (!func) {
        LOG_ERROR("REG", "No function named '%s'", name);
        return 2;
    }
    va_list va;
    va_start(va, count);
    int ret = func(&va);
    va_end(va);
    return ret;
}
